@@ -2,16 +2,14 @@ import json
 import os
 
 import docker
-from dagster import Field, StringSource, check
-from dagster.core.host_representation import ExternalPipeline
-from dagster.core.launcher.base import RunLauncher
-from dagster.core.storage.pipeline_run import PipelineRun
+from dagster import Field, Permissive, StringSource, check
+from dagster.core.launcher.base import LaunchRunContext, RunLauncher
+from dagster.core.storage.tags import DOCKER_IMAGE_TAG
 from dagster.grpc.types import ExecuteRunArgs
 from dagster.serdes import ConfigurableClass, serialize_dagster_namedtuple
 from docker_image import reference
 
 DOCKER_CONTAINER_ID_TAG = "docker/container_id"
-DOCKER_IMAGE_TAG = "docker/image"
 
 
 class DockerRunLauncher(RunLauncher, ConfigurableClass):
@@ -25,14 +23,43 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
             docker container.
         network (Optional[str]): Name of the network this container to which to connect the
             launched container at creation time.
+        container_kwargs(Optional[Dict[str, Any]]): Additional kwargs to pass into
+            containers.create. See https://docker-py.readthedocs.io/en/stable/containers.html
+            for the full list of available options.
     """
 
-    def __init__(self, inst_data=None, image=None, registry=None, env_vars=None, network=None):
+    def __init__(
+        self,
+        inst_data=None,
+        image=None,
+        registry=None,
+        env_vars=None,
+        network=None,
+        container_kwargs=None,
+    ):
         self._inst_data = inst_data
         self._image = image
         self._registry = registry
         self._env_vars = env_vars
         self._network = network
+        self._container_kwargs = check.opt_dict_param(
+            container_kwargs, "container_kwargs", key_type=str
+        )
+
+        if "image" in self._container_kwargs:
+            raise Exception(
+                "'image' cannot be used in 'container_kwargs'. Use the 'image' config key instead."
+            )
+
+        if "environment" in self._container_kwargs and env_vars:
+            raise Exception(
+                "Cannot specify both `env_vars` in DockerRunLauncher config and `environment` in `container_kwargs`. Choose one or the other."
+            )
+
+        if "network" in self._container_kwargs and network:
+            raise Exception(
+                "Cannot specify both `network` in DockerRunLauncher config and `network` in `container_kwargs`. Choose one or the other."
+            )
 
         super().__init__()
 
@@ -63,10 +90,17 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
                 description="The list of environment variables names to forward to the docker container",
             ),
             "network": Field(
-                str,
+                StringSource,
                 is_required=False,
                 description="Name of the network this container to which to connect the launched container at creation time",
             ),
+            "container_kwargs": Field(
+                Permissive(),
+                is_required=False,
+                description="key-value pairs that can be passed into containers.create. See "
+                "https://docker-py.readthedocs.io/en/stable/containers.html for the full list "
+                "of available options.",
+            ),
         }
 
     @staticmethod
@@ -83,11 +117,13 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
             )
         return client
 
-    def launch_run(self, run, external_pipeline):
-        check.inst_param(run, "run", PipelineRun)
-        check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
+    def launch_run(self, context: LaunchRunContext) -> None:
 
-        docker_image = external_pipeline.get_python_origin().repository_origin.container_image
+        run = context.pipeline_run
+
+        pipeline_code_origin = context.pipeline_code_origin
+
+        docker_image = pipeline_code_origin.repository_origin.container_image
 
         if not docker_image:
             docker_image = self._image
@@ -107,7 +143,7 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
 
         input_json = serialize_dagster_namedtuple(
             ExecuteRunArgs(
-                pipeline_origin=external_pipeline.get_python_origin(),
+                pipeline_origin=pipeline_code_origin,
                 pipeline_run_id=run.run_id,
                 instance_ref=self._instance.get_ref(),
             )
@@ -128,6 +164,7 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
                 detach=True,
                 environment=docker_env,
                 network=self._network,
+                **self._container_kwargs,
             )
 
         except docker.errors.ImageNotFound:
@@ -138,6 +175,7 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
                 detach=True,
                 environment=docker_env,
                 network=self._network,
+                **self._container_kwargs,
             )
 
         self._instance.report_engine_event(
@@ -156,8 +194,6 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
 
         container.start()
 
-        return run
-
     def _get_container(self, run):
         if not run or run.is_finished:
             return None
